import argparse
import asyncio
import importlib

from flask import Flask, jsonify, request, send_file
from flask_cors import CORS
from urllib.parse import urljoin

from .validate import validate_template

DESCRIPTION = '''
This utility is for the development of Ellacard group video templates. It will
build and serve your templates locally using a lightweight flask server. It is
designed to work with the group video template development tools here:
https://ellacard.com/design

Documentation: https://ellacard.com/design-video-template-docs
'''

parser = argparse.ArgumentParser(description=DESCRIPTION, formatter_class=argparse.RawDescriptionHelpFormatter)
parser.add_argument('template', help='path to the python template module')
parser.add_argument('--port', type=int, default=5000, help='the port on which to run the server')
args = parser.parse_args()


loop = asyncio.get_event_loop()
app = Flask('ellacard-serve-template')
CORS(app)


def _build_template_module():
  with open(args.template) as file:
    template_spec = importlib.util.spec_from_loader('template', loader=None)
    template_module = importlib.util.module_from_spec(template_spec)
    exec(file.read(), template_module.__dict__)
    validate_template(template_module)
    return template_module


@app.route('/get-file', methods=[ 'GET' ])
def get_file():
  return send_file(args.template)


@app.route('/get-json', methods=[ 'GET' ])
def get_json():
  template = _build_template_module()
  return jsonify({ 'name': template.name, 'description': template.description, 'sections': template.sections })


@app.route('/build', methods=[ 'POST' ])
def build():
  project = request.get_json()
  template = _build_template_module()
  project_build = loop.run_until_complete(template.build(project))
  return jsonify(project_build)


app.run(host='0.0.0.0', port=args.port)
